@@ -39,11 +39,9 @@ import PyQt4.QtCore as QtCore
 from electrum.i18n import _, set_language
 from electrum.plugins import run_hook
 from electrum import SimpleConfig, Wallet, WalletStorage
-from electrum.paymentrequest import InvoiceStore
-from electrum.contacts import Contacts
 from electrum.synchronizer import Synchronizer
 from electrum.verifier import SPV
-from electrum.util import DebugMem, UserCancelled
+from electrum.util import DebugMem, UserCancelled, InvalidPassword
 from electrum.wallet import Abstract_Wallet
 from installwizard import InstallWizard, GoBack
 
@@ -89,9 +87,6 @@ class ElectrumGui:
         self.app = QApplication(sys.argv)
         self.app.installEventFilter(self.efilter)
         self.timer = Timer()
-        # shared objects
-        self.invoices = InvoiceStore(self.config)
-        self.contacts = Contacts(self.config)
         # init tray
         self.dark_icon = self.config.get("dark_icon", False)
         self.tray = QSystemTrayIcon(self.tray_icon(), None)
@@ -159,13 +154,10 @@ class ElectrumGui:
                 w.bring_to_top()
                 break
         else:
-            try:
-                wallet = self.daemon.load_wallet(path)
-            except BaseException as e:
-                QMessageBox.information(None, _('Error'), str(e), _('OK'))
-                return
-            if wallet is None:
-                wizard = InstallWizard(self.config, self.app, self.plugins, path)
+            wallet = self.daemon.load_wallet(path, None)
+            if not wallet:
+                storage = WalletStorage(path)
+                wizard = InstallWizard(self.config, self.app, self.plugins, storage)
                 wallet = wizard.run_and_get_wallet()
                 if not wallet:
                     return
