@@ -285,19 +285,16 @@ def get_user_id(storage):
     return long_id, short_id
 
 def make_xpub(xpub, s):
-    _, _, _, c, cK = deserialize_xpub(xpub)
+    version, _, _, _, c, cK = deserialize_xpub(xpub)
     cK2, c2 = bitcoin._CKD_pub(cK, c, s)
-    xpub2 = ("0488B21E" + "00" + "00000000" + "00000000").decode("hex") + c2 + cK2
-    return EncodeBase58Check(xpub2)
-
+    return bitcoin.serialize_xpub(version, c2, cK2)
 
 def make_billing_address(wallet, num):
     long_id, short_id = wallet.get_user_id()
     xpub = make_xpub(billing_xpub, long_id)
-    _, _, _, c, cK = deserialize_xpub(xpub)
+    version, _, _, _, c, cK = deserialize_xpub(xpub)
     cK, c = bitcoin.CKD_pub(cK, c, num)
-    address = public_key_to_bc_address( cK )
-    return address
+    return bitcoin.public_key_to_p2pkh(cK)
 
 
 class TrustedCoinPlugin(BasePlugin):
@@ -335,7 +332,7 @@ class TrustedCoinPlugin(BasePlugin):
         return True
 
     def make_seed(self):
-        return Mnemonic('english').make_seed(num_bits=128, prefix=SEED_PREFIX)
+        return Mnemonic('english').make_seed(seed_type='2fa', num_bits=128)
 
     @hook
     def do_clear(self, window):
@@ -380,11 +377,11 @@ class TrustedCoinPlugin(BasePlugin):
         xprv1, xpub1, xprv2, xpub2 = self.xkeys_from_seed(seed, passphrase)
         k1 = keystore.from_xprv(xprv1)
         k2 = keystore.from_xpub(xpub2)
-        wizard.request_password(run_next=lambda pw: self.on_password(wizard, pw, k1, k2))
+        wizard.request_password(run_next=lambda pw, encrypt: self.on_password(wizard, pw, encrypt, k1, k2))
 
-    def on_password(self, wizard, password, k1, k2):
+    def on_password(self, wizard, password, encrypt, k1, k2):
         k1.update_password(None, password)
-        wizard.storage.put('use_encryption', bool(password))
+        wizard.storage.set_password(password, encrypt)
         wizard.storage.put('x1/', k1.dump())
         wizard.storage.put('x2/', k2.dump())
         wizard.storage.write()
@@ -427,12 +424,12 @@ class TrustedCoinPlugin(BasePlugin):
 
     def on_choice(self, wizard, seed, passphrase, x):
         if x == 'disable':
-            f = lambda pw: wizard.run('on_restore_pw', seed, passphrase, pw)
+            f = lambda pw, encrypt: wizard.run('on_restore_pw', seed, passphrase, pw, encrypt)
             wizard.request_password(run_next=f)
         else:
             self.create_keystore(wizard, seed, passphrase)
 
-    def on_restore_pw(self, wizard, seed, passphrase, password):
+    def on_restore_pw(self, wizard, seed, passphrase, password, encrypt):
         storage = wizard.storage
         xprv1, xpub1, xprv2, xpub2 = self.xkeys_from_seed(seed, passphrase)
         k1 = keystore.from_xprv(xprv1)
@@ -445,8 +442,8 @@ class TrustedCoinPlugin(BasePlugin):
         long_user_id, short_id = get_user_id(storage)
         xpub3 = make_xpub(signing_xpub, long_user_id)
         k3 = keystore.from_xpub(xpub3)
-        storage.put('use_encryption', bool(password))
         storage.put('x3/', k3.dump())
+        storage.set_password(password, encrypt)
         wizard.wallet = Wallet_2fa(storage)
         wizard.create_addresses()
 
@@ -524,7 +521,7 @@ class TrustedCoinPlugin(BasePlugin):
         challenge = r.get('challenge')
         message = 'TRUSTEDCOIN CHALLENGE: ' + challenge
         def f(xprv):
-            _, _, _, c, k = deserialize_xprv(xprv)
+            _, _, _, _, c, k = deserialize_xprv(xprv)
             pk = bip32_private_key([0, 0], k, c)
             key = regenerate_key(pk)
             compressed = is_compressed(pk)
